@@ -49,6 +49,8 @@
 #define BACKUP_DIRECTORY "/var/lib/nvidia"
 #define BACKUP_LOG       (BACKUP_DIRECTORY "/log")
 
+#define RMMOD_MODULE_NAME "nvidia"
+
 /*
  * XXX when uninstalling should we remove directories that were
  * created by our installation?
@@ -117,9 +119,10 @@ typedef struct {
 } BackupInfo;
 
 
-
 static BackupInfo *read_backup_log_file(Options *op);
 
+static void free_backup_info(BackupInfo *b);
+
 static int check_backup_log_entries(Options *op, BackupInfo *b);
 
 static int do_uninstall(Options *op);
@@ -498,7 +501,7 @@ static int do_uninstall(Options *op)
     BackupLogEntry *e;
     BackupInfo *b;
     int i, len, ok;
-    char *tmpstr;
+    char *tmpstr, *cmd;
     float percent;
 
     static const char existing_installation_is_borked[] = 
@@ -656,8 +659,19 @@ static int do_uninstall(Options *op)
         /* XXX what to do if this fails?... nothing */
     }
 
+    /*
+     * attempt to unload the kernel module, but don't abort if this fails: the
+     * kernel may not have been configured with support for module unloading
+     * (Linux 2.6) or the user might have unloaded it themselves.
+     */
+    cmd = nvstrcat(op->utils[RMMOD], " ", RMMOD_MODULE_NAME, NULL);
+    run_command(op, cmd, NULL, FALSE, 0, TRUE);
+    nvfree(cmd);
+
     run_distro_hook(op, "post-uninstall");
 
+    free_backup_info(b);
+
     return TRUE;
     
 } /* do_uninstall() */
@@ -758,6 +772,8 @@ static BackupInfo *read_backup_log_file(Options *op)
         b->e = (BackupLogEntry *)
             nvrealloc(b->e, sizeof(BackupLogEntry) * b->n);
 
+        memset(&b->e[b->n - 1], 0, sizeof(BackupLogEntry));
+
         e = &b->e[b->n - 1];
         e->num = num;
         e->filename = filename;
@@ -840,6 +856,33 @@ static BackupInfo *read_backup_log_file(Options *op)
 } /* read_backup_log_file() */
 
 
+
+/*
+ * free_backup_info() - free the data associated with BackupInfo
+ */
+
+static void free_backup_info(BackupInfo *b)
+{
+    int i;
+
+    if (!b) return;
+    
+    nvfree(b->version);
+    nvfree(b->description);
+    
+    for (i = 0; i < b->n; i++) {
+        nvfree(b->e[i].filename);
+        nvfree(b->e[i].target);
+    }
+
+    nvfree((char *) b->e);
+    
+    nvfree((char *) b);
+
+} /* free_backup_info() */
+
+
+
 /*
  * check_backup_log_entries() - for each backup log entry, perform
  * some basic sanity checks.  Set the 'ok' field to FALSE if a
@@ -1068,7 +1111,7 @@ int check_for_existing_driver(Options *op, Package *p)
     int ret = FALSE;
     int localRet;
 
-    if (!check_for_existing_rpms(op)) return FALSE;
+    if (!check_for_existing_rpms(op)) goto done;
 
     localRet = get_installed_driver_version_and_descr(op, &version, &descr);
 
@@ -1078,7 +1121,7 @@ int check_for_existing_driver(Options *op, Package *p)
                      "'--kernel-module-only' option can only be used "
                      "to install the NVIDIA kernel module on top of an "
                      "existing driver installation.");
-            return FALSE;
+            goto done;
         } else {
             if (strcmp(p->version, version) != 0) {
                 ui_error(op, "The '--kernel-module-only' option can only be "
@@ -1087,9 +1130,10 @@ int check_for_existing_driver(Options *op, Package *p)
                          "version.  The existing driver installation is "
                          "%s, but the kernel module is %s.\n",
                          version, p->version);
-                return FALSE;
+                goto done;
             } else {
-                return TRUE;
+                ret = TRUE;
+                goto done;
             }
         }
     }
@@ -1116,9 +1160,8 @@ int check_for_existing_driver(Options *op, Package *p)
                    "continue? ('no' will abort installation)",
                    version, p->version)) {
         
-        free(descr);
         ui_log(op, "Installation aborted.");
-        return FALSE;
+        goto done;
     }
     
     ret = TRUE;
@@ -1221,7 +1264,9 @@ int test_installed_files(Options *op)
     
     ret = sanity_check_backup_log_entries(op, b);
     
-    /* XXX should free resources associated with b */
+    /* free resources associated with b */
+    
+    free_backup_info(b);
 
     return ret;
     
@@ -1238,7 +1283,7 @@ int find_installed_file(Options *op, char *filename)
 {
     BackupInfo *b;
     BackupLogEntry *e;
-    int i;
+    int i, ret = FALSE;
     
     if ((b = read_backup_log_file(op)) == NULL) return FALSE;
 
@@ -1249,13 +1294,14 @@ int find_installed_file(Options *op, char *filename)
             /* XXX should maybe compare inodes rather than
                filenames? */
 
-            return TRUE;
+            ret =  TRUE;
+            break;
         }
     }
 
-    /* XXX need to free b */
+    free_backup_info(b);
 
-    return FALSE;
+    return ret;
 
 } /* find_installed_file() */
 
