@@ -736,8 +736,8 @@ int test_kernel_module(Options *op, Package *p)
                  "built against the wrong or improperly configured "
                  "kernel sources, with a version of gcc that differs "
                  "from the one used to build the target kernel, or "
-                 "if a driver such as rivafb/nvidiafb is present and "
-                 "prevents the NVIDIA kernel module from obtaining "
+                 "if a driver such as rivafb, nvidiafb, or nouveau is present "
+                 "and prevents the NVIDIA kernel module from obtaining "
                  "ownership of the NVIDIA graphics device(s).\n\n"
                  "Please see the log entries 'Kernel module load "
                  "error' and 'Kernel messages' at the end of the file "
@@ -1756,15 +1756,15 @@ static char *guess_kernel_module_filename(Options *op)
     minor = atoi(dot0);
     
     if ((major > 2) || ((major == 2) && (minor > 4))) {
-        return "nvidia.ko";
+        return nvstrdup("nvidia.ko");
     } else {
-        return "nvidia.o";
+        return nvstrdup("nvidia.o");
     }
 
  fail:
     ui_error (op, "Unable to determine if kernel is 2.6.0 or greater from "
               "uname string '%s'; assuming the kernel module filename is "
               "'nvidia.o'.", str);
-    return "nvidia.o";
+    return nvstrdup("nvidia.o");
     
 } /* guess_kernel_module_filename() */
